"""
:authors: Ivan Kudinov
:license: The MIT License (MIT) see LICENSE file
:copyright: (c) 2023 Ivan Kudinov
"""

from .db import *
from .config import *
from .protocols import *
from .gps_server import *
from .connections import *

__version__ = "0.0.1"



import asyncio
import datetime
from config.settings import settings
import logging
from connections import handle_connection
from db.postgres import postgres_init, postgres_get_info

logging.basicConfig(level=logging.DEBUG)


async def _init_db():
    """
    Инициализация БД
    """
    logger = logging.getLogger("gpswe::init_db")
    if (
        settings.POSTGRES_DB
        and settings.POSTGRES_PORT
        and settings.POSTGRES_HOST
        and settings.POSTGRES_PASSWORD
        and settings.POSTGRES_USER
    ):
        dsl = {
            "database": settings.POSTGRES_DB,
            "user": settings.POSTGRES_USER,
            "password": settings.POSTGRES_PASSWORD,
            "host": settings.POSTGRES_HOST,
            "port": settings.POSTGRES_PORT,
        }
        logger.info("Start initialized db")
        await postgres_init(dsl)
    else:
        logger.warning("Not all DB data was provided")


def init_db():
    """
    Ассинхронный запуск инициализации БД
    """
    loop = asyncio.new_event_loop()
    loop.run_until_complete(_init_db())


async def _start_server():
    """
    Запуск сервера
    """
    logger = logging.getLogger("gpswe::start_server")
    if (
        settings.POSTGRES_DB
        and settings.POSTGRES_PORT
        and settings.POSTGRES_HOST
        and settings.POSTGRES_PASSWORD
        and settings.POSTGRES_USER
    ):
        server = await asyncio.start_server(
            handle_connection, settings.HOST, settings.PORT
        )
        async with server:
            logger.info(f"Server started at: {settings.HOST}:{settings.PORT}")
            await server.serve_forever()
    else:
        logger.warning("Not all DB data was provided")


def start_server():
    """
    Ассинхронный запуск сервера
    """
    loop = asyncio.new_event_loop()
    loop.run_until_complete(_start_server())


async def _get_coordinates(
    imei: str, from_datetime: datetime.datetime, to_datetime: datetime.datetime
):
    """
    Получение доп. данных в промежутке дат
    """
    logger = logging.getLogger("gpswe::get_coordinates")
    if (
        settings.POSTGRES_DB
        and settings.POSTGRES_PORT
        and settings.POSTGRES_HOST
        and settings.POSTGRES_PASSWORD
        and settings.POSTGRES_USER
    ):
        dsl = {
            "database": settings.POSTGRES_DB,
            "user": settings.POSTGRES_USER,
            "password": settings.POSTGRES_PASSWORD,
            "host": settings.POSTGRES_HOST,
            "port": settings.POSTGRES_PORT,
        }
        logger.info("Get extend GPS data")
        return await postgres_get_info(dsl, imei, from_datetime, to_datetime)
    else:
        logger.warning("Not all DB data was provided")


def get_coordinates(
    imei: str, from_datetime: datetime.datetime, to_datetime: datetime.datetime
):
    """
    Ассинхронный запуск получение доп. данных в промежутке дат
    """
    loop = asyncio.new_event_loop()
    return loop.run_until_complete(_get_coordinates(imei, from_datetime, to_datetime))



import logging
from protocols.wialon import wialon_protocol
from protocols.egts import egts_protocol
from config.settings import settings

logging.basicConfig(level=logging.DEBUG)


async def handle_connection(reader, writer):
    """
    Прослушивание сервера и обработка полученных данных
    """
    logger = logging.getLogger("gpswe::handle_connection")
    addr = writer.get_extra_info("peername")
    dsl = {
        "database": settings.POSTGRES_DB,
        "user": settings.POSTGRES_USER,
        "password": settings.POSTGRES_PASSWORD,
        "host": settings.POSTGRES_HOST,
        "port": settings.POSTGRES_PORT,
    }
    logger.info(f"Connected by {addr}")
    while True:
        try:
            data = await reader.read(settings.BUFF_SIZE)
        except ConnectionError:
            logger.info(f"Client suddenly closed while receiving from {addr}")
            break
        if not data:
            break
        data = data.upper()
        try:
            answer = data.decode("utf-8").split("\r\n")
            for item in answer:
                # Обработка Wialon протокола
                send_data = await wialon_protocol(item, addr, dsl)
                # Отправление ответа клиенту
                try:
                    writer.write(bytearray(send_data.encode("utf-8")))
                    await writer.drain()
                except ConnectionError:
                    logger.info("Client suddenly closed, cannot send")
                    break
        except UnicodeDecodeError:
            answer = data
            # Обработка EGTS протокола
            send_data = await egts_protocol(answer, dsl)
            # Отправление ответа клиенту
            try:
                writer.write(send_data)
                await writer.drain()
            except ConnectionError:
                logger.info("Client suddenly closed, cannot send")
                break
    writer.close()
    await writer.wait_closed()
    logger.info(f"Disconnected by: {addr}")



#!/user/bin/env python

from io import open
from setuptools import setup

"""
:authors: Ivan Kudinov
:license: The MIT License (MIT) see LICENSE file
:copyright: (c) 2023 Ivan Kudinov
"""

version = "0.0.1"

long_description = '''Python library for parse, save and work 
                    with Wialon and EGTS data'''

setup(
    name="gpswe",
    version=version,

    author="Ivan Kudinov",
    author_email="marvel.2012@mail.ru",

    description=(
        u'Python library for parse, save and work ' 
        u'with Wialon and EGTS data'
    ),
    long_description=long_description,
    long_description_content_type="text/markdown",

    url="https://github.com/KudinovIvan/gpswe",
    download_url="https://github.com/KudinovIvan/gpswe/archive/v{}.zip".format(
        version
    ),

    license='The MIT License (MIT) see LICENSE file',

    packages=['gpswe'],
    install_requires=["asyncio", "pydantic", "crcmod", "asyncpg", "geopy"],

    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Intended Audience :: End Users/Desktop',
        'Intended Audience :: Developers',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Programming Language :: Python :: Implementation :: CPython',
    ]
)